@@ -4,9 +4,10 @@ import inspect
 import io
 import json
 import logging
+import os
+import uuid
 
 import jsonschema
-import uuid
 
 from .connection import ConnectionCache
 from .dispatcher import Stdout, Http
@@ -15,9 +16,8 @@ from .exceptions import (
     ServerException,
     LoggedException,
     ConnectionTestException,
-    PluginException,
 )
-
+from .metrics import MetricsBuilder
 
 message_output_type = {
     "action_start": "action_event",
@@ -28,21 +28,21 @@ message_output_type = {
 
 class Workflow(object):
     def __init__(
-        self,
-        shortOrgId=None,
-        orgProductToken=None,
-        uiHostUrl=None,
-        jobId=None,
-        stepId=None,
-        versionId=None,
-        nextStepId=None,
-        nextEdgeId=None,
-        triggerId=None,
-        jobExecutionContextId=None,
-        time=None,
-        connectionTestId=None,
-        connectionTestTimeout=None,
-        workflowId=None,
+            self,
+            shortOrgId=None,
+            orgProductToken=None,
+            uiHostUrl=None,
+            jobId=None,
+            stepId=None,
+            versionId=None,
+            nextStepId=None,
+            nextEdgeId=None,
+            triggerId=None,
+            jobExecutionContextId=None,
+            time=None,
+            connectionTestId=None,
+            connectionTestTimeout=None,
+            workflowId=None,
     ):
         """
         Worflow object for the Meta Class
@@ -133,14 +133,14 @@ class Plugin(object):
     """A Komand Plugin."""
 
     def __init__(
-        self,
-        name="",
-        vendor="",
-        description="",
-        version="",
-        connection=None,
-        custom_encoder=None,
-        custom_decoder=None,
+            self,
+            name="",
+            vendor="",
+            description="",
+            version="",
+            connection=None,
+            custom_encoder=None,
+            custom_decoder=None,
     ):
         self.name = name
         self.vendor = vendor
@@ -172,16 +172,17 @@ class Plugin(object):
         """ add a new task"""
         self.tasks[task.name] = task
 
-    @staticmethod
     def envelope(
-        message_type,
-        input_message,
-        log,
-        success,
-        output,
-        error_message,
-        state,
-        ex: None,
+            self,
+            message_type,
+            input_message,
+            log,
+            success,
+            output,
+            error_message,
+            state,
+            ex: None,
+            is_test: False
     ):
         """
         Creates an output message of a step's execution.
@@ -194,6 +195,7 @@ class Plugin(object):
         :param error_message: An error message if an error was thrown
         :param state: The state of task_event. Only applicable to tasks.
         :param ex: An error that was thrown
+        :param is_test: whether or not the step is part of a Connection Test
         :return: An output message
         """
 
@@ -210,12 +212,34 @@ class Plugin(object):
             output_message["output"] = output
         else:
             output_message["error"] = error_message
-            if ex and isinstance(ex, ConnectionTestException):
-                output_message["exception"] = {
-                    "cause": ex.cause,
-                    "assistance": ex.assistance,
-                    "data": ex.data,
-                }
+
+            if ex:
+                if isinstance(ex, ConnectionTestException):
+                    output_message["exception"] = {
+                        "cause": ex.cause,
+                        "assistance": ex.assistance,
+                        "data": ex.data,
+                    }
+
+                if is_test and not isinstance(ex, ConnectionTestException):
+                    output_message["exception"] = {
+                        "cause": "Plugin connection test failed.",
+                        "assistance": "See error log for more details.",
+                        "data": ex.__repr__()
+                    }
+
+                # Build the metrics blob to attach to the output payload
+                # Only supported in cloud environments right now
+                if os.environ.get("PLUGIN_RUNTIME_ENVIRONMENT") == "cloud":
+                    metrics_builder = MetricsBuilder(plugin_name=self.name,
+                                                     plugin_version=self.version,
+                                                     plugin_vendor=self.vendor,
+                                                     input_message=input_message,
+                                                     exception_=ex,
+                                                     workflow_id=self.connection.meta.workflow.workflowId,
+                                                     org_id=self.connection.meta.workflow.shortOrgId)
+                    output_message["metrics"] = metrics_builder.build()
+
         return {"body": output_message, "version": "v1", "type": message_type}
 
     def marshal(self, msg, fd):
@@ -327,7 +351,7 @@ class Plugin(object):
         logger.addHandler(stream_handler)
 
         success = True
-        ex = None
+        caught_exception = None
         output = None
         out_type = None
         state = None
@@ -385,35 +409,29 @@ class Plugin(object):
                         is_test,
                         is_debug,
                     )
-        except ClientException as e:
+        except (ClientException, ServerException, Exception) as e:
             success = False
-            ex = e
-            logger.exception(e)
-        except ServerException as e:
-            success = False
-            ex = e
-            logger.exception(e)
-        except Exception as e:
-            success = False
-            ex = e
+            caught_exception = e
             logger.exception(e)
         finally:
-            output = Plugin.envelope(
+            output = self.envelope(
                 out_type,
                 input_message,
                 log_stream.getvalue(),
                 success,
                 output,
-                str(ex),
+                str(caught_exception),
                 state,
-                ex,
+                caught_exception,
+                is_test
             )
+            print(f"Output is: {output}")
             if not success:
-                raise LoggedException(ex, output)
+                raise LoggedException(caught_exception, output)
             return output
 
     def start_step(
-        self, message_body, step_key, logger, log_stream, is_test=False, is_debug=False
+            self, message_body, step_key, logger, log_stream, is_test=False, is_debug=False
     ):
         """
         Starts an action.
